// Visual Micro is in vMicro>General>Tutorial Mode
// 
/*
    Name:       PWMBox.ino
    Created:	8/28/2018 8:15:12 AM
    Author:     ArtWin7Dev-PC\ArtWin7Dev
*/
//Sample using LiquidCrystal library 
#include <LiquidCrystal.h> 
// select the pins used on the LCD panel 
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
// define some values used by the panel and buttons 
int lcd_key = 0;
int adc_key_in = 0;
#define btnRIGHT  0 
#define btnUP     1 
#define btnDOWN   2 
#define btnLEFT   3 
#define btnSELECT 4 
#define btnNONE   5 

volatile uint32_t CaptureCountA, CaptureCountB, TimerCount, OnTime, OutTime;
volatile boolean CaptureFlag;
volatile int  ScaleCurrent;
uint32_t  LastButMilli;

int16_t Values[12800];


// read the buttons 
int read_LCD_buttons()
{
	adc_key_in = analogRead(0);      // read the value from the sensor  
									 // my buttons when read are centered at these valies: 0, 144, 329, 504, 741 
									 // we add approx 50 to those values and check to see if we are close 
	if (adc_key_in > 1000) return btnNONE; // We make this the 1st option for speed reasons since it will be the most likely result 
	if (adc_key_in < 50)   return btnRIGHT;
	if (adc_key_in < 300)  return btnUP;
	if (adc_key_in < 600)  return btnDOWN;
	if (adc_key_in < 800)  return btnLEFT;
	if (adc_key_in < 790)  return btnSELECT;
	return btnNONE;  // when all others fail, return this... 
}
void setup()
{
	lcd.begin(16, 2);              // start the library 
	lcd.setCursor(0, 0);
	lcd.print("PWM Scaler %   "); // print a simple message 
	lcd.setCursor(0, 1);
	lcd.print("Out%    -In:    ");
	ScaleCurrent = 100; //100% of incoming to outgoing
	LastButMilli = millis();
	lcd.setCursor(4, 1);
	lcd.print(ScaleCurrent);

	//setup for pwm monitoring
//reading pin A7 as incoming PWM
/**************************************************************************************/
/*                        Capture PWM frequency and duty cycle                        */
/*      Hook a jumper between pin A10 and pin A7 for selftest                        */
/**************************************************************************************/
	//Serial.begin(250000);

 	                           
	//setup timer interrupt to measure incoming PWM signal, interrupt on loads

	PMC->PMC_PCER0 |= PMC_PCER0_PID28;               // Timer Counter 0 channel 1 IS TC1
	TC0->TC_CHANNEL[1].TC_CCR = TC_CCR_CLKDIS;       // disable internal clocking while setup regs
	TC0->TC_CHANNEL[1].TC_CMR = TC_CMR_TCCLKS_TIMER_CLOCK1 // capture mode, MCK/2, clk on rising edge                           
		| TC_CMR_ABETRG              // TIOA is used as the external trigger
		| TC_CMR_LDRA_RISING         // load RA on rising edge of trigger input
		| TC_CMR_LDRB_FALLING;       // load RB on falling edge of trigger input
	TC0->TC_CHANNEL[1].TC_IER |= TC_IER_LDRAS | TC_IER_LDRBS; // Trigger interruption on Load RA and load RB
	TC0->TC_CHANNEL[1].TC_CCR = TC_CCR_SWTRG | TC_CCR_CLKEN; // Reset TC counter and enable
	NVIC_EnableIRQ(TC1_IRQn);                             // Enable TC1 interrupts

	//turn on DAC1  as PWM output for laser, set to 2us tickle
	//turn on A10 as test output set to 50% duty, vary in loop for testing
	//jumper a7 to a10 for testing
	//PWM set to 5Khz

	REG_PMC_PCER1 |= PMC_PCER1_PID36;                     // Enable PWM 
	REG_PIOB_ABSR |= PIO_ABSR_P16 | PIO_ABSR_P19;         // Set PWM pin perhipheral type A or B, in this case B
	REG_PIOB_PDR |= PIO_PDR_P16 | PIO_ABSR_P19;         // Set PWM pin to an output
	REG_PWM_CLK = PWM_CLK_PREA(0) | PWM_CLK_DIVA(2);      // Set the PWM clock rate to 84MHz (84MHz/1) 
	
	//pin dac1 -- output, set to 2us
	REG_PWM_CMR0 = PWM_CMR_CPRE_CLKA;                     // Enable single slope PWM and set the clock source as CLKA
	REG_PWM_CPRD0 = 8400;                                 // Set the PWM frequency 84MHz/40kHz = 2100 
	REG_PWM_CDTY0 = 128;                                  // Set the PWM duty cycle 50% (2100/2=1050)

	//pin a10 as test input , jumper to pin 2 for reading
	REG_PWM_CMR3 = PWM_CMR_CPRE_CLKA;
	REG_PWM_CPRD3 = 8400;                                 // Set the PWM frequency 84MHz/40kHz = 2100 
	REG_PWM_CDTY3 = 4100;
    //enable PWM on Dac1 and A10
	REG_PWM_ENA = PWM_ENA_CHID0 | PWM_ENA_CHID3;
	//100 = tickle,
	//8000 = 100%
	//use REG_PWM_CDTYUPD0 to set new value to laser control.
    
	//set %
	lcd.setCursor(13, 0);
	lcd.print(ScaleCurrent);
	
	//build tables
	for (int x = 0; x < 100; x++)
	{
		int tablespot = x * 128;
		double min = 200;
		double target = x + 1; //this is the target for this 128 series of integers
		target = ((target / 100) * 8192);
		//so target is between 0 - 8192
		if (target < min) target = min;
		double num = (target - min) / 128.0;
		for (int y = 0; y < 128; y++)
		{
			//we need to move smoothly from 0 - target in 128 entries
			int spot = tablespot + y;
			double val = (y * num) + min;
			int16_t ent = val;
			if (ent < 128)
				ent = 128;
			if (y < 3 )
				ent = 128;
			Values[spot] = ent;
		}
	}


}
 
int duty = 128;
int lastmilli = 0;
bool Aborted = false;
uint32_t LastInt = 0;

void loop()
{ 
	uint32_t curmill = millis();
	lcd_key = read_LCD_buttons();  // read the buttons 
	if (micros() - LastInt > 250)
	{
		//kill the output
		REG_PWM_CDTYUPD0 = 128;
		OutTime = 128;
	}

	if (curmill - lastmilli > 500)
	{
		lastmilli = curmill;
		duty += 128;
		if (duty > 8192)
			duty = 128;
		REG_PWM_CDTYUPD3 = duty;
	}

	if ( curmill - LastButMilli > 150)
	{
		switch (lcd_key)               // depending on which button was pushed, we perform an action 
		{
		case btnRIGHT:
		   {
			//lcd.print("RIGHT ");
			break;
		   }
		case btnLEFT:
		   {
			//lcd.print("LEFT   ");
			break;
		   }
		case btnUP:
		   {
			if (ScaleCurrent < 100)
				ScaleCurrent++;
			LastButMilli = curmill;
			lcd.setCursor(13, 0);
			lcd.print(ScaleCurrent);
			lcd.print(" ");
			break;
		   }
		case btnDOWN:
		   {
			if (ScaleCurrent > 5)
				ScaleCurrent--;
			LastButMilli = curmill;
			lcd.setCursor(13, 0);
			lcd.print(ScaleCurrent);
			lcd.print(" ");
			break;
		   }

		}
	}
	//now read the pwm channel
	if (1)//CaptureFlag) 
	{
		
		lcd.setCursor(12, 1);
		lcd.print(OnTime);
		lcd.print("  ");
		lcd.setCursor(4, 1);
		if(!Aborted)
		  lcd.print(OutTime);
		else
		  lcd.print("Abrt");
		lcd.print(" ");
		Aborted = false;
		CaptureFlag = 0;
	}

}
 
// Note that you could either test status register by polling in loop()
void TC1_Handler() 
{

	LastInt = micros();
  //Registers A and B (RA and RB) are used as capture registers. They are loaded with
  //the counter value TC_CV when a programmable event occurs on the signal TIOA.
	TimerCount = TC0->TC_CHANNEL[1].TC_CV;            // save the timer counter register, for testing

	uint32_t status = TC0->TC_CHANNEL[1].TC_SR;       // Read & Save satus register -->Clear status register

	// If TC_SR_LOVRSRA is set, RA or RB have been loaded at least twice without any read
	// of the corresponding register since the last read of the Status Register,
	// We are losing some values,trigger of TC_Handler is not fast enough !!
	if ((status & TC_SR_LOVRS) == TC_SR_LOVRS)
	{
		Aborted = true;
		OutTime = 128;
		REG_PWM_CDTYUPD0 = OutTime;
		//return;
	}

	if ((status & TC_SR_LDRAS) == TC_SR_LDRAS)  //rising edge
	{  // If ISR is fired by LDRAS then ....

		CaptureCountA = TC0->TC_CHANNEL[1].TC_RA;        // get data from capture register A for TC0 channel 1
		CaptureCountB = 0;
		CaptureFlag = 0;
	}
	else if ((status & TC_SR_LDRBS) == TC_SR_LDRBS) //falling edge
	{ // If ISR is fired by LDRBS then ....
		if (!CaptureCountB && CaptureCountA)
		{
			CaptureCountB = TC0->TC_CHANNEL[1].TC_RB;         // get data from caputre register B for TC0 channel 1
			OnTime = CaptureCountB - CaptureCountA;
			CaptureCountA = 0;
			CaptureFlag = 1;
			//now calc percentage; //just divide by 128 for 1 segment..
			int test = OnTime >> 6; //divide by 64
			if (test > 127)
				test = 0;
			int16_t spot = (ScaleCurrent - 1) << 7;
			OutTime = Values[spot + test];
           	REG_PWM_CDTYUPD0 = OutTime;
		}
		else
		{
			CaptureCountB = CaptureCountA = 0;
		}

	}



}

